# 2010 - Copyright by Bee <http://www.honeybeenet.altervista.org>.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import gedit
import pygtk
pygtk.require('2.0')
import gtk
from gettext import gettext as _
import os
import tempfile
import time
import subprocess
import pickle

def error_message(message):
    def msg_on_close_destroy(button,response_id,dialog):
        if response_id == gtk.RESPONSE_CLOSE: 
            dialog.destroy()

    msg_dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL, buttons = gtk.BUTTONS_CLOSE, type = gtk.MESSAGE_ERROR, message_format = message)
    msg_dialog.connect('response', msg_on_close_destroy, msg_dialog)
    msg_dialog.run()

class WindowControl:
    def __init__(self, plugin, window):
        self._window = window
        self._plugin = plugin
        self._insert_menu()

    def deactivate(self):
        self._remove_menu()
        self._window = self._plugin = self._action_group = None

    def _insert_menu(self):
        manager = self._window.get_ui_manager()

        self._action_group = gtk.ActionGroup("beesuPluginActions")
        self._action_group.add_actions([("beesuPlugin", None, 
                                        _("Open as \"root\""),
                                         None, 
                                        _("Open this document as \"root\" in gedit with beesu"),
                                         self.beesu)])

        manager.insert_action_group(self._action_group, -1)

        ui_str = """<ui>
            <menubar name="MenuBar">
                <menu name="ToolsMenu" action="Tools">
                    <placeholder name="ToolsOps_2">
                        <menuitem name="beesuPlugin" action="beesuPlugin"/>
                    </placeholder>
                    </menu>
            </menubar>
        </ui>
        """

        self._ui_id = manager.add_ui_from_string(ui_str)

    def _remove_menu(self):
        manager = self._window.get_ui_manager()
        manager.remove_ui(self._ui_id)
        manager.remove_action_group(self._action_group)
        manager.ensure_update()

    def update_ui(self):
        self._action_group.set_sensitive(self._window.get_active_document() != None)

    def beesu(self, action):
        tab = self._window.get_active_tab()
        doc = self._window.get_active_document()

        filename = None
        if doc:
            filename = doc.get_uri()

        if not self.beesu_exec(filename):
            return

        if tab:
            if doc == tab.get_document():
                if doc.get_readonly() or doc.is_untouched():
                    self._window.close_tab(tab)
                    if self._window.get_active_document() == None and self._window.get_active_tab() == None:
                        self._window.destroy()
        
    def beesu_exec(self, filename):
        tmp = tempfile.NamedTemporaryFile(delete = False)
        lockfile = tmp.name
        tmp.close()
        #['beesu', '-l', '-c', '/usr/libexec/gedit-beesu-plugin','gedit-beesu-plugin','/usr/bin/gedit','gedit',lockfile]
        shell_args = ['beesu', '-l','/usr/libexec/gedit-beesu-plugin','gedit-beesu-plugin','gedit','gedit',lockfile]
        if filename != None and filename != '':
            shell_args.append(filename)
        try:
            subprocess.Popen(shell_args, stderr=open(os.devnull)).wait()
        except:
            error_message(_("Error running beesu: " + sys.exc_info()[0]))
            return False

        if os.path.isfile(lockfile):
            os.remove(lockfile)
            return False

        return True

class beesuPlugin(gedit.Plugin):
    def __init__(self):
        gedit.Plugin.__init__(self)
        self._instances = {}

    def activate(self, window):
        self._instances[window] = WindowControl(self, window)

    def deactivate(self, window):
        self._instances[window].deactivate()
        del self._instances[window]

    def update_ui(self, window):
        self._instances[window].update_ui()

    def is_configurable(self):
        return False  

